require 'go_run'
require 'open3'

describe GoRun do
  before(:each) do
    @binary = File.expand_path("../../bin/run", __FILE__)

    # Make sure no local Runfiles interfere with tests
    pwd_runfile = File.expand_path("./Runfile.yml", `pwd`.strip)
    expect(File.file? pwd_runfile).to be(false)
  end

  context "when no Runfile is provided" do
    it "should print an error" do
      stdout, status = Open3.capture2(@binary)
      expect(stdout).to include("Runfile not found")
    end
  end

  context "when no arguments are provided" do
    it "should print an error" do
      runfile_path = File.expand_path("spec/fixtures/runfiles/Runfile.yml")
      stdout, status = Open3.capture2(@binary, "-r", runfile_path)
      expect(stdout).to include("You must choose at least one target")
    end
  end

  context "when arguments are provided" do
    it "should run the targets" do
      runfile_path = File.expand_path("spec/fixtures/runfiles/Runfile.yml")
      stdout, status = Open3.capture2(@binary, "-r", runfile_path, "dev", "deploy")
      expect(stdout).to include("eu-west-1")
    end
  end

  context "when the targeted commands fail" do
    it "should exit with a non-zero code" do
      runfile_path = File.expand_path("spec/fixtures/runfiles/Runfile-with-failures.yml")
      stdout, status = Open3.capture2(@binary, "-r", runfile_path, "dev", "deploy")
      expect(stdout).to include("eu-west-1")
      expect(status.exitstatus).to be(1)
    end
  end
end
