require 'go_run/parser'

describe Parser do
  it "should raise an error if it is not a hash" do
    file_path = File.expand_path("../../spec/fixtures/parser/array.yml", __FILE__)
    expect{Parser.new(file_path)}.to raise_error(RuntimeError, "Configuration error: config is not a hash")
  end

  it "should raise an error if a target is not an array" do
    file_path = File.expand_path("../../spec/fixtures/parser/target-not-an-array.yml", __FILE__)
    expect{Parser.new(file_path)}.to raise_error(RuntimeError, "Configuration error: each target should be an array of strings")
  end

  it "should raise an error if a target is not an array of strings" do
    file_path = File.expand_path("../../spec/fixtures/parser/target-not-an-array-of-strings.yml", __FILE__)
    expect{Parser.new(file_path)}.to raise_error(RuntimeError, "Configuration error: each line of a target should be a string")
  end

  it "should raise an error if a target line does not start with a keyword" do
    file_path = File.expand_path("../../spec/fixtures/parser/target-line-invalid.yml", __FILE__)
    expect{Parser.new(file_path)}.to raise_error(RuntimeError, "Configuration error: target lines should match /^(GROUP|SET|REQUIRE|CALL|RUN)/")
  end

  it "should build an array of commands" do
    file_path = File.expand_path("../../spec/fixtures/parser/commands.yml", __FILE__)
    config = Parser.new(file_path).config
    expect(config["deploy"].has_key? "commands").to be(true)

    commands = config["deploy"]["commands"]
    expect(commands.length).to be(2)
    expect(commands[0]).to eq('echo "deploying..."')
    expect(commands[1]).to eq('date')
  end

  it "should build an array of required env vars and their defaults" do
    file_path = File.expand_path("../../spec/fixtures/parser/env-vars.yml", __FILE__)
    config = Parser.new(file_path).config
    expect(config["dev"].has_key? "requires").to be(true)
    expect(config["dev"]["requires"].length).to be(2)
    expect(config["dev"]["requires"][0].has_key? "key").to be(true)
    expect(config["dev"]["requires"][1].has_key? "default").to be(true)
    expect(config["dev"]["requires"][1]["default"]).to eq('eu-west-1')
  end

  it "should build an array of env var key-value pairs to set" do
    file_path = File.expand_path("../../spec/fixtures/parser/set-env-vars.yml", __FILE__)
    config = Parser.new(file_path).config
    expect(config["dev"].has_key? "set").to be(true)
    expect(config["dev"]["set"].length).to be(1)
    expect(config["dev"]["set"][0].has_key? "key").to be(true)
    expect(config["dev"]["set"][0].has_key? "value").to be(true)
    expect(config["dev"]["set"][0]["key"]).to eq('ENVIRONMENT')
    expect(config["dev"]["set"][0]["value"]).to eq('dev')
  end

  it "should build an array of calls to other targets" do
    file_path = File.expand_path("../../spec/fixtures/parser/calls.yml", __FILE__)
    config = Parser.new(file_path).config
    expect(config["dev"].has_key? "call").to be(true)
    expect(config["dev"]["call"].length).to be(1)
    expect(config["dev"]["call"][0].has_key? "target").to be(true)
    expect(config["dev"]["call"][0]["target"]).to eq('some-other-target')
  end

  it "should build a group of other targets to call" do
    file_path = File.expand_path("../../spec/fixtures/parser/group.yml", __FILE__)
    config = Parser.new(file_path).config
    expect(config["dev-deploy"].has_key? "group").to be(true)
    expect(config["dev-deploy"]["group"].length).to be(2)
    expect(config["dev-deploy"]["group"][0]).to eq('dev')
    expect(config["dev-deploy"]["group"][1]).to eq('deploy')
  end
end
